# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.views.generic import DetailView, ListView, TemplateView

from parler.views import TranslatableSlugMixin

from .models import Simple, Untranslated


class SimpleRootView(TemplateView):
    template = 'test_addon/empty.html'


class SimpleListView(ListView):
    model = Simple
    http_method_names = ['get', ]


class SimpleDetailView(TranslatableSlugMixin, DetailView):
    model = Simple
    view_url_name = 'thing-detail'


class UntranslatedDetailView(DetailView):
    model = Untranslated
    view_url_name = 'untranslated-detail'



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import resolve, reverse
from django.test import TransactionTestCase

from aldryn_translation_tools.utils import (
    get_admin_url,
    get_object_from_request,
)

from test_addon.models import Simple, Untranslated

from . import SimpleTransactionTestCase


class TestUtils(TransactionTestCase):

    def test_get_admin_url(self):
        # With pattern args, but no URL parameters
        change_action = 'auth_user_change'
        args = [1, ]
        kwargs = {}
        url = get_admin_url(change_action, args, **kwargs)
        self.assertIn('/admin/auth/user/1/', url)

        # With pattern args and a single URL parameter
        kwargs = {'alpha': 'beta', }
        url = get_admin_url(change_action, args, **kwargs)
        self.assertIn('/admin/auth/user/1/?alpha=beta', url)

        # With pattern args and two URL parameters
        kwargs = {'alpha': 'beta', 'gamma': 'delta', }
        url = get_admin_url(change_action, args, **kwargs)
        self.assertIn('/admin/auth/user/1/?alpha=beta&gamma=delta', url)

        # With pattern args and 3 URL parameters
        kwargs = {'a': 'b', 'g': 'd', 'e': 'z', }
        url = get_admin_url(change_action, args, **kwargs)
        self.assertIn('/admin/auth/user/1/?a=b&e=z&g=d', url)

        # With pattern args and numerical URL params
        kwargs = {'a': 1, 'g': 2, 'e': 3, }
        url = get_admin_url(change_action, args, **kwargs)
        self.assertIn('/admin/auth/user/1/?a=1&e=3&g=2', url)

        # With pattern args and odd-typed URL params
        kwargs = {'a': [], 'g': {}, 'e': None}
        url = get_admin_url(change_action, args, **kwargs)
        self.assertIn('/en/admin/auth/user/1/?a=%5B%5D&e=None&g=%7B%7D', url)

        # No pattern args...
        add_action = 'auth_user_add'
        kwargs = {'groups': 1, }
        url = get_admin_url(add_action, **kwargs)
        self.assertIn('/admin/auth/user/add/?groups=1', url)

        # No pattern args and no kwargs
        add_action = 'auth_user_add'
        url = get_admin_url(add_action)
        self.assertIn('/admin/auth/user/add/', url)


class TestToolbarHelpers(SimpleTransactionTestCase):

    def test_get_obj_from_request(self):
        """ Test that we can get the object from the request. """
        self.simple1.set_current_language('en')
        slug_url = self.simple1.get_absolute_url('en')
        pk_url = slug_url.replace(
            '/{0}/'.format(self.simple1.slug),
            '/{0}/'.format(self.simple1.pk))

        for url in [slug_url, pk_url, ]:
            request = self.request_factory.get(url)
            request.LANGUAGE_CODE = 'en'
            request.current_page = self.page
            request.user = self.user
            request.resolver_match = resolve(request.path)

            simple = get_object_from_request(Simple, request)
            self.assertTrue(simple.pk, self.simple1.pk)

    def test_get_obj_from_empty_request(self):
        """ Test that we get None if the request doesn't contain an object. """
        request = self.request_factory.get(reverse('simple:simple-root'))
        request.LANGUAGE_CODE = 'en'
        request.current_page = self.page
        request.user = self.user
        request.resolver_match = resolve(request.path)
        simple = get_object_from_request(Simple, request)
        self.assertIsNone(simple)

    def test_get_untranslated_obj_from_request(self):
        """ Test that we can get a non-translated object from the request. """
        slug_url = self.untranslated1.get_absolute_url()
        pk_url = slug_url.replace(
            '/{0}/'.format(self.untranslated1.slug),
            '/{0}/'.format(self.untranslated1.pk))

        for url in [slug_url, pk_url, ]:
            request = self.request_factory.get(url)
            request.LANGUAGE_CODE = 'en'
            request.current_page = self.page
            request.user = self.user
            request.resolver_match = resolve(request.path)
            untranslated = get_object_from_request(Untranslated, request)
            self.assertTrue(untranslated.pk, self.untranslated1.pk)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from parler.managers import TranslatableManager, TranslatableQuerySet


class SimpleQuerySet(TranslatableQuerySet):
    pass


class SimpleManager(TranslatableManager):
    queryset_class = SimpleQuerySet

    def get_queryset(self):
        qs = SimpleQuerySet(self.model, using=self.db)
        return qs



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import (
    SimpleDetailView,
    SimpleListView,
    SimpleRootView,
    UntranslatedDetailView,
)

urlpatterns = patterns(
    '',
    url(r'^empty-view', SimpleRootView.as_view(), name='simple-root'),
    url(r'^simple/$', SimpleListView.as_view(), name='simple-list'),

    # NOTE: We are allowing access by slug and pk here.
    url(r'^simple/(?P<pk>\d+)/$', SimpleDetailView.as_view(),
        name='simple-detail'),
    url(r'^simple/(?P<slug>\w[-\w]*)/$', SimpleDetailView.as_view(),
        name='simple-detail'),

    url(r'^untranslated/(?P<pk>\d+)/$', UntranslatedDetailView.as_view(),
        name='untranslated-detail'),
    url(r'^untranslated/(?P<slug>\w[-\w]*)/$', UntranslatedDetailView.as_view(),
        name='untranslated-detail'),
)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.core.urlresolvers import NoReverseMatch
from django.contrib.sitemaps import Sitemap
from django.utils import translation


class I18NSitemap(Sitemap):
    """
    A helper class that supports translated sitemaps.

    In sitemaps.py (example):

        class ThingsSitemap(I18NSitemap):
            changefreq = "weekly"
            priority = 0.75

            def items(self):
                # Make sure to limit items to those translated into the language
                # set in self.language.
                return Thing.objects.published().translated(self.language).all()

    Then, in urls.py, when setting up the sitemaps dictionary, use as normal,
    but instantiate the class with a language.

        sitemaps = {
            # Specify non-translated apps as usual
            'janky_app': JankyAppSitemap,

            # Translated Apps
            'things-en': ThingsSitemap('en'),
            'things-fr': ThingsSitemap('fr'),
            'things-de': ThingsSitemap('de'),
            ...
        }

    Continue as normal.
    """

    def __init__(self, language=None):
        """
        Override's Sitemap's constructor to accept a language code as
        a parameter.
        """
        super(I18NSitemap, self).__init__()
        self.language = language or settings.LANGUAGES[0][0]

    def location(self, item):
        """
        Overrides Sitemap.location() to utilise the language set in
        self.language.
        """
        with translation.override(self.language):
            try:
                return item.get_absolute_url()
            except NoReverseMatch:  # pragma: no cover
                # Note, if we did our job right in items(), this
                # shouldn't happen at all, but just in case...
                return ''
